package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.enums.RespCodeEnum;
import cn.wisenergy.common.utils.exception.BaseCustomException;
import cn.wisenergy.mapper.*;
import cn.wisenergy.model.app.Actives;
import cn.wisenergy.model.app.Users;
import cn.wisenergy.model.dto.UserSimpleInfoDto;
import cn.wisenergy.model.dto.UsersInfoDto;
import cn.wisenergy.service.app.TestUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Authotr:陈奇
 * @QQ1799796883
 */
@Service
@Slf4j
public class TestUserServiceImpl implements TestUserService {

    @Autowired
    private UsersMapper usersMapper;

    @Autowired
    private UserRechargeMapper userRechargeMapper;

    @Autowired
    private UserWithdrawsMapper userWithdrawsMapper;

    @Autowired
    private ActivesMapper activesMapper;

    @Autowired
    private UsdtOrderDetailsMapper usdtOrderDetailsMapper;

    private final String MONEYISEMPTY = "0";

    /**
     * 查询下级用户的基本信息
     *
     * @param userId 用户id
     * @return
     */
    @Override
    public List<UserSimpleInfoDto> getDownUserInfoById(Long userId) {
        log.info("TestUserServiceImpl[]getDownUserInfoById[]input.param.userId:" + userId);
        // 1.判断用户是否存在
        if (null==userId) {
            userId = 1L;
        }
        Users user = usersMapper.selectById(userId);
        if (null == user) {
            throw new BaseCustomException(RespCodeEnum.EMPLOYEE_IS_NOT_EXIST_ERROR);
        }
        // 2.获取用户伞下所有用户的信息
        List<UserSimpleInfoDto> downUsersById = usersMapper.getDownUsersById(userId);
        return downUsersById;
    }

    /**
     * 查询用户详细信息
     *
     * @param userId 用户id
     * @return
     */

    @Override
    public UsersInfoDto getUserInfoById(Long userId) {
        log.info("TestUserServiceImpl[]getUserInfoById[]input.param.userId:" + userId);
        // 1.判断用户是否存在
        Users user = userIsEmpty(userId);
        // 2.获取用户详细信息
        Actives userInfo = activesMapper.getByUserId(userId);
        // 3.查询最深层级数和伞下人员总和
        long startTime=System.currentTimeMillis();
        UsersInfoDto userInfoDto = usersMapper.getDownLevelAndTotalPeopleByUserId(userId);
        userInfoDto.setBottom(userInfoDto.getBottom() - user.getRank());
        long endTime=System.currentTimeMillis();
        // 4.查询用户充值和提现总额
        Double totalRecharge = userRechargeMapper.getTotalRecharge(userId);
        Double totalWithdrawal = userWithdrawsMapper.getTotalWithdrawal(userId);
        // 5.计算OTC买入卖出
        Double buyTotal = usdtOrderDetailsMapper.getBuyTotal(userId);
        Double saleTotal = usdtOrderDetailsMapper.getSaleTotal(userId);

        // 5.将查询出来的信息整合到dto类
        userInfoDto.setUserId(userId);
        BeanUtils.copyProperties(user, userInfoDto);
        if (null != userInfo) {
            BeanUtils.copyProperties(userInfo, userInfoDto);
        }
        userInfoDto.setTotalRecharge(totalRecharge == null ? MONEYISEMPTY : totalRecharge.toString());
        userInfoDto.setTotalWithdrawal(totalWithdrawal == null ? MONEYISEMPTY : totalWithdrawal.toString());
        userInfoDto.setBuyTotal(buyTotal == null ? MONEYISEMPTY : buyTotal.toString());
        userInfoDto.setSaleTotal(saleTotal == null ? MONEYISEMPTY : saleTotal.toString());

        System.out.println("程序运行时间： "+(endTime-startTime)+"ms");
        return userInfoDto;
    }


    /**
     * 判断用户是否存在，存在并返回user信息
     *
     * @param userId
     * @return
     */
    private Users userIsEmpty(Long userId) {
        if (null == userId) {
            throw new BaseCustomException(RespCodeEnum.INPUT_PARAMETER_ISEMPTY);
        }
        Users user = usersMapper.selectById(userId);
        if (null == user) {
            throw new BaseCustomException(RespCodeEnum.EMPLOYEE_IS_NOT_EXIST_ERROR);
        }
        return user;
    }
}
