package cn.wisenergy.web.admin.controller.app;

import cn.wisenergy.common.constant.RedisConsts;
import cn.wisenergy.common.utils.R;
import cn.wisenergy.model.app.Users;
import cn.wisenergy.model.dto.UsersInfoDto;
import cn.wisenergy.model.vo.UserVo;
import cn.wisenergy.service.app.UserService;
import cn.wisenergy.web.common.BaseController;
import cn.wisenergy.web.shiro.JwtUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


/**
 * @author 86187
 */
@Api(tags = "用户管理")
@RestController
@Slf4j
public class UserController extends BaseController {

    @Autowired
    private UserService userService;

    @Autowired
    private JwtUtil jwtUtil;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;


    /**
     * shiro登录
     *
     * @return/user
     */
    @ApiOperation(value = "获取token接口", notes = "获取token接口", httpMethod = "POST")
    @PostMapping(value = "/user/login")
    public R<String> login(Long id) {
        if (null == id) {
            return R.error("入参为空!");
        }

        //用户信息
        Users users = userService.getById(id);

        // 创建token
        String token = jwtUtil.generateToken(users);
        // 保存Redis
        redisTemplate.opsForValue().set(RedisConsts.JWT_ACCESS_TOKEN + token, token);
        return R.ok(token);
    }


    @ApiOperation(value = "获取用户顶级/下一级", notes = "获取用户顶级/下一级", httpMethod = "GET")
    @ApiImplicitParam(name = "userId", value = "用户id,空值查询顶级，其他查询下一级", dataType = "int")
    @GetMapping("/user/getLastUser")
    public R<List<UserVo>> getLastUser(Long userId) {
        return userService.getLastUser(userId);
    }

    @ApiOperation(value = "获取用户详细信息", notes = "获取用户详细信息", httpMethod = "GET")
    @ApiImplicitParam(name = "userId", value = "用户id", dataType = "int")
    @GetMapping("/user/getUserInfo")
    public R<UsersInfoDto> getUserInfo(Long userId) {
        return userService.getByUserInfoById(userId);
    }

    @ApiOperation(value = "获取用户直连树结构", notes = "获取用户直连树结构", httpMethod = "GET")
    @ApiImplicitParam(name = "userId", value = "用户id", dataType = "int")
    @GetMapping("/user/getLinkTree")
    public R<List<UserVo>> getLinkTree(Long userId) {
        return userService.getLinkTree(userId);
    }
}
